import random
import unittest

import requests
from .fixtures import APITestCase


class TestApi(APITestCase):
    def test_pod_start_conflict(self):
        """Verify issue #8865"""

        pod_name = list()
        pod_name.append(f"Pod_{random.getrandbits(160):x}")
        pod_name.append(f"Pod_{random.getrandbits(160):x}")

        r = requests.post(
            self.uri("/pods/create"),
            json={
                "name": pod_name[0],
                "no_infra": False,
                "portmappings": [{"host_ip": "127.0.0.1", "host_port": 8889, "container_port": 89}],
            },
        )
        self.assertEqual(r.status_code, 201, r.text)
        r = requests.post(
            self.uri("/containers/create"),
            json={
                "pod": pod_name[0],
                "image": "quay.io/libpod/alpine:latest",
                "command": ["top"],
            },
        )
        self.assertEqual(r.status_code, 201, r.text)

        r = requests.post(
            self.uri("/pods/create"),
            json={
                "name": pod_name[1],
                "no_infra": False,
                "portmappings": [{"host_ip": "127.0.0.1", "host_port": 8889, "container_port": 89}],
            },
        )
        self.assertEqual(r.status_code, 201, r.text)
        r = requests.post(
            self.uri("/containers/create"),
            json={
                "pod": pod_name[1],
                "image": "quay.io/libpod/alpine:latest",
                "command": ["top"],
            },
        )
        self.assertEqual(r.status_code, 201, r.text)

        r = requests.post(self.uri(f"/pods/{pod_name[0]}/start"))
        self.assertEqual(r.status_code, 200, r.text)

        r = requests.post(self.uri(f"/pods/{pod_name[1]}/start"))
        self.assertEqual(r.status_code, 409, r.text)

        start = r.json()
        self.assertGreater(len(start["Errs"]), 0, r.text)


if __name__ == "__main__":
    unittest.main()



from flask import Flask
import os
app = Flask(__name__)

@app.route('/')
def hello():
    f = open("/data/message", "w")
    f.write(os.getenv("PODMAN_MSG"))
    f.close()
    return "done"

if __name__ == '__main__':
	app.run(host='0.0.0.0')



import random
import unittest

import requests

from .fixtures import APITestCase


class NetworkTestCase(APITestCase):
    # TODO Need to support Docker-py order of network/container creates
    def test_connect(self):
        """Create network and container then connect to network"""
        net_default = requests.post(
            self.podman_url + "/v1.40/networks/create", json={"Name": "TestDefaultNetwork"}
        )
        self.assertEqual(net_default.status_code, 201, net_default.text)

        create = requests.post(
            self.podman_url + "/v1.40/containers/create?name=postCreateConnect",
            json={
                "Cmd": ["top"],
                "Image": "alpine:latest",
                "NetworkDisabled": False,
                # FIXME adding these 2 lines cause: (This is sampled from docker-py)
                #   "network already exists","message":"container
                #  01306e499df5441560d70071a54342611e422a94de20865add50a9565fd79fb9 is already connected to CNI
                #  network \"TestDefaultNetwork\": network already exists"
                # "HostConfig": {"NetworkMode": "TestDefaultNetwork"},
                # "NetworkingConfig": {"EndpointsConfig": {"TestDefaultNetwork": None}},
                # FIXME These two lines cause:
                # CNI network \"TestNetwork\" not found","message":"error configuring network namespace for container
                # 369ddfa7d3211ebf1fbd5ddbff91bd33fa948858cea2985c133d6b6507546dff: CNI network \"TestNetwork\" not
                # found"
                # "HostConfig": {"NetworkMode": "TestNetwork"},
                # "NetworkingConfig": {"EndpointsConfig": {"TestNetwork": None}},
                # FIXME no networking defined cause: (note this error is from the container inspect below)
                # "internal libpod error","message":"network inspection mismatch: asked to join 2 CNI network(s) [
                # TestDefaultNetwork podman], but have information on 1 network(s): internal libpod error"
            },
        )
        self.assertEqual(create.status_code, 201, create.text)
        self.assertId(create.content)

        payload = create.json()
        start = requests.post(self.podman_url + f"/v1.40/containers/{payload['Id']}/start")
        self.assertEqual(start.status_code, 204, start.text)

        connect = requests.post(
            self.podman_url + "/v1.40/networks/TestDefaultNetwork/connect",
            json={"Container": payload["Id"]},
        )
        self.assertEqual(connect.status_code, 200, connect.text)
        self.assertEqual(connect.text, "OK\n")

        inspect = requests.get(f"{self.podman_url}/v1.40/containers/{payload['Id']}/json")
        self.assertEqual(inspect.status_code, 200, inspect.text)

        payload = inspect.json()
        self.assertFalse(payload["Config"].get("NetworkDisabled", False))

        self.assertEqual(
            "TestDefaultNetwork",
            payload["NetworkSettings"]["Networks"]["TestDefaultNetwork"]["NetworkID"],
        )
        # TODO restore this to test, when joining multiple networks possible
        # self.assertEqual(
        #     "TestNetwork",
        #     payload["NetworkSettings"]["Networks"]["TestNetwork"]["NetworkID"],
        # )
        # TODO Need to support network aliases
        # self.assertIn(
        #     "test_post_create",
        #     payload["NetworkSettings"]["Networks"]["TestNetwork"]["Aliases"],
        # )

    def test_create(self):
        """Create network and connect container during create"""
        net = requests.post(
            self.podman_url + "/v1.40/networks/create", json={"Name": "TestNetwork"}
        )
        self.assertEqual(net.status_code, 201, net.text)

        create = requests.post(
            self.podman_url + "/v1.40/containers/create?name=postCreate",
            json={
                "Cmd": ["date"],
                "Image": "alpine:latest",
                "NetworkDisabled": False,
                "HostConfig": {"NetworkMode": "TestNetwork"},
            },
        )
        self.assertEqual(create.status_code, 201, create.text)
        self.assertId(create.content)

        payload = create.json()
        inspect = requests.get(f"{self.podman_url}/v1.40/containers/{payload['Id']}/json")
        self.assertEqual(inspect.status_code, 200, inspect.text)

        payload = inspect.json()
        self.assertFalse(payload["Config"].get("NetworkDisabled", False))
        self.assertEqual(
            "TestNetwork",
            payload["NetworkSettings"]["Networks"]["TestNetwork"]["NetworkID"],
        )
    def test_inspect(self):
        name = f"Network_{random.getrandbits(160):x}"
        create = requests.post(self.podman_url + "/v1.40/networks/create", json={"Name": name})
        self.assertEqual(create.status_code, 201, create.text)
        self.assertId(create.content)

        net = create.json()
        self.assertIsInstance(net, dict)
        self.assertNotEqual(net["Id"], name)
        ident = net["Id"]

        ls = requests.get(self.podman_url + "/v1.40/networks")
        self.assertEqual(ls.status_code, 200, ls.text)

        networks = ls.json()
        self.assertIsInstance(networks, list)

        found = False
        for net in networks:
            if net["Name"] == name:
                found = True
                break
        self.assertTrue(found, f"Network '{name}' not found")

        inspect = requests.get(self.podman_url + f"/v1.40/networks/{ident}?verbose=false&scope=local")
        self.assertEqual(inspect.status_code, 200, inspect.text)


    def test_crud(self):
        name = f"Network_{random.getrandbits(160):x}"

        # Cannot test for 0 existing networks because default "podman" network always exists

        create = requests.post(self.podman_url + "/v1.40/networks/create", json={"Name": name})
        self.assertEqual(create.status_code, 201, create.text)
        self.assertId(create.content)

        net = create.json()
        self.assertIsInstance(net, dict)
        self.assertNotEqual(net["Id"], name)
        ident = net["Id"]

        ls = requests.get(self.podman_url + "/v1.40/networks")
        self.assertEqual(ls.status_code, 200, ls.text)

        networks = ls.json()
        self.assertIsInstance(networks, list)

        found = False
        for net in networks:
            if net["Name"] == name:
                found = True
                break
        self.assertTrue(found, f"Network '{name}' not found")

        inspect = requests.get(self.podman_url + f"/v1.40/networks/{ident}")
        self.assertEqual(inspect.status_code, 200, inspect.text)
        self.assertIsInstance(inspect.json(), dict)

        inspect = requests.delete(self.podman_url + f"/v1.40/networks/{ident}")
        self.assertEqual(inspect.status_code, 204, inspect.text)
        inspect = requests.get(self.podman_url + f"/v1.40/networks/{ident}")
        self.assertEqual(inspect.status_code, 404, inspect.text)

        # network prune
        prune_name = f"Network_{random.getrandbits(160):x}"
        prune_create = requests.post(
            self.podman_url + "/v1.40/networks/create", json={"Name": prune_name}
        )
        self.assertEqual(create.status_code, 201, prune_create.text)

        prune = requests.post(self.podman_url + "/v1.40/networks/prune")
        self.assertEqual(prune.status_code, 200, prune.text)
        self.assertTrue(prune_name in prune.json()["NetworksDeleted"])


if __name__ == "__main__":
    unittest.main()



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))

import re

# -- Project information -----------------------------------------------------

project = "Podman"
copyright = "2019, team"
author = "team"


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = ["myst_parser"]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []

master_doc = "index"

# Configure smartquotes to only transform quotes and ellipses, not dashes
smartquotes_action = "qe"


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "alabaster"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

html_css_files = [
    "custom.css",
]

# -- Extension configuration -------------------------------------------------

# IMPORTANT: explicitly unset the extensions, by default dollarmath is enabled.
# We use the dollar sign as text and do not want it to be interpreted as math expression.
myst_enable_extensions = []


def convert_markdown_title(app, docname, source):
    # Process markdown files only
    docpath = app.env.doc2path(docname)
    if docpath.endswith(".md"):
        # Convert pandoc title line into eval_rst block for myst_parser
        source[0] = re.sub(r"^% (.*)", r"```{title} \g<1>\n```", source[0])


def setup(app):
    app.connect("source-read", convert_markdown_title)



import subprocess
import sys
import time
import unittest

from docker import DockerClient

from test.python.docker import Podman, constant
from test.python.docker.compat import common


class TestSystem(unittest.TestCase):
    podman = None  # initialized podman configuration for tests
    service = None  # podman service instance
    topContainerId = ""

    def setUp(self):
        super().setUp()
        self.client = DockerClient(base_url="tcp://127.0.0.1:8080", timeout=15)

        TestSystem.podman.restore_image_from_cache(self.client)
        TestSystem.topContainerId = common.run_top_container(self.client)

    def tearDown(self):
        common.remove_all_containers(self.client)
        common.remove_all_images(self.client)
        self.client.close()
        return super().tearDown()

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        TestSystem.podman = Podman()
        TestSystem.service = TestSystem.podman.open(
            "system", "service", "tcp:127.0.0.1:8080", "--time=0"
        )
        # give the service some time to be ready...
        time.sleep(2)

        returncode = TestSystem.service.poll()
        if returncode is not None:
            raise subprocess.CalledProcessError(returncode, "podman system service")

    @classmethod
    def tearDownClass(cls):
        TestSystem.service.terminate()
        stdout, stderr = TestSystem.service.communicate(timeout=0.5)
        if stdout:
            sys.stdout.write("\nImages Service Stdout:\n" + stdout.decode("utf-8"))
        if stderr:
            sys.stderr.write("\nImAges Service Stderr:\n" + stderr.decode("utf-8"))

        TestSystem.podman.tear_down()
        return super().tearDownClass()

    def test_Info(self):
        info = self.client.info()
        self.assertIsNotNone(info)
        self.assertEqual(info["RegistryConfig"]["IndexConfigs"]["localhost:5000"]["Secure"], False)
        self.assertEqual(info["RegistryConfig"]["IndexConfigs"]["localhost:5000"]["Mirrors"], ["mirror.localhost:5000"])

    def test_info_container_details(self):
        info = self.client.info()
        self.assertEqual(info["Containers"], 1)
        self.client.containers.create(image=constant.ALPINE)
        info = self.client.info()
        self.assertEqual(info["Containers"], 2)

    def test_version(self):
        version = self.client.version()
        self.assertIsNotNone(version["Platform"]["Name"])



import unittest

import requests
from .fixtures import APITestCase


class ManifestTestCase(APITestCase):
    def test_manifest_409(self):
        r = requests.post(self.uri("/manifests/create"), params={"name": "ThisIsAnInvalidImage"})
        self.assertEqual(r.status_code, 400, r.text)


if __name__ == "__main__":
    unittest.main()



import collections
import io
import os
import subprocess
import sys
import time
import unittest

from docker import DockerClient, errors
from docker.errors import APIError

from test.python.docker import Podman
from test.python.docker.compat import common, constant


class TestImages(unittest.TestCase):
    podman = None  # initialized podman configuration for tests
    service = None  # podman service instance

    def setUp(self):
        super().setUp()
        self.client = DockerClient(base_url="tcp://127.0.0.1:8080", timeout=15)

        TestImages.podman.restore_image_from_cache(self.client)

    def tearDown(self):
        common.remove_all_images(self.client)
        self.client.close()
        return super().tearDown()

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        TestImages.podman = Podman()
        TestImages.service = TestImages.podman.open(
            "system", "service", "tcp:127.0.0.1:8080", "--time=0"
        )
        # give the service some time to be ready...
        time.sleep(2)

        returncode = TestImages.service.poll()
        if returncode is not None:
            raise subprocess.CalledProcessError(returncode, "podman system service")

    @classmethod
    def tearDownClass(cls):
        TestImages.service.terminate()
        stdout, stderr = TestImages.service.communicate(timeout=0.5)
        if stdout:
            sys.stdout.write("\nImages Service Stdout:\n" + stdout.decode("utf-8"))
        if stderr:
            sys.stderr.write("\nImAges Service Stderr:\n" + stderr.decode("utf-8"))

        TestImages.podman.tear_down()
        return super().tearDownClass()

    def test_tag_valid_image(self):
        """Validates if the image is tagged successfully"""
        alpine = self.client.images.get(constant.ALPINE)
        self.assertTrue(alpine.tag("demo", constant.ALPINE_SHORTNAME))

        alpine = self.client.images.get(constant.ALPINE)
        for t in alpine.tags:
            self.assertIn("alpine", t)

    # @unittest.skip("doesn't work now")
    def test_retag_valid_image(self):
        """Validates if name updates when the image is retagged"""
        alpine = self.client.images.get(constant.ALPINE)
        self.assertTrue(alpine.tag("demo", "rename"))

        alpine = self.client.images.get(constant.ALPINE)
        self.assertNotIn("demo:test", alpine.tags)

    def test_list_images(self):
        """List images"""
        self.assertEqual(len(self.client.images.list()), 1)

        # Add more images
        self.client.images.pull(constant.BB)
        self.assertEqual(len(self.client.images.list()), 2)
        self.assertEqual(len(self.client.images.list(all=True)), 2)

        # List images with filter
        self.assertEqual(len(self.client.images.list(filters={"reference": "alpine"})), 1)

    def test_search_image(self):
        """Search for image"""
        for r in self.client.images.search("alpine"):
            # registry matches if string is in either one
            self.assertIn("alpine", r["Name"]+" "+r["Description"].lower())

    def test_search_bogus_image(self):
        """Search for bogus image should throw exception"""
        try:
            r = self.client.images.search("bogus/bogus")
        except:
            return
        self.assertTrue(len(r) == 0)

    def test_remove_image(self):
        """Remove image"""
        # Check for error with wrong image name
        with self.assertRaises(errors.NotFound):
            self.client.images.remove("dummy")
        self.assertEqual(len(self.client.images.list()), 1)

        self.client.images.remove(constant.ALPINE)
        self.assertEqual(len(self.client.images.list()), 0)

    def test_image_history(self):
        """Image history"""
        img = self.client.images.get(constant.ALPINE)
        history = img.history()
        image_id = img.id[7:] if img.id.startswith("sha256:") else img.id

        found = False
        for change in history:
            found |= image_id in change.values()
        self.assertTrue(found, f"image id {image_id} not found in history")

    def test_get_image_exists_not(self):
        """Negative test for get image"""
        with self.assertRaises(errors.NotFound):
            response = self.client.images.get("image_does_not_exists")
            collections.deque(response)

    def test_save_image(self):
        """Export Image"""
        image = self.client.images.pull(constant.BB)

        file = os.path.join(TestImages.podman.image_cache, "busybox.tar")
        with open(file, mode="wb") as tarball:
            for frame in image.save(named=True):
                tarball.write(frame)
        sz = os.path.getsize(file)
        self.assertGreater(sz, 0)

    def test_load_image(self):
        """Import|Load Image"""
        self.assertEqual(len(self.client.images.list()), 1)

        image = self.client.images.pull(constant.BB)
        file = os.path.join(TestImages.podman.image_cache, "busybox.tar")
        with open(file, mode="wb") as tarball:
            for frame in image.save():
                tarball.write(frame)

        with open(file, mode="rb") as saved:
            _ = self.client.images.load(saved)

        self.assertEqual(len(self.client.images.list()), 2)

    def test_load_corrupt_image(self):
        """Import|Load Image failure"""
        tarball = io.BytesIO("This is a corrupt tarball".encode("utf-8"))
        with self.assertRaises(APIError):
            self.client.images.load(tarball)

    def test_build_image(self):
        labels = {"apple": "red", "grape": "green"}
        _ = self.client.images.build(
            path="test/python/docker/build_labels", labels=labels, tag="labels", isolation="default"
        )
        image = self.client.images.get("labels")
        self.assertEqual(image.labels["apple"], labels["apple"])
        self.assertEqual(image.labels["grape"], labels["grape"])


if __name__ == "__main__":
    # Setup temporary space
    unittest.main()



import json
import os
import shlex
import signal
import string
import subprocess
import sys
import time
import unittest
from collections.abc import Iterable
from multiprocessing import Process

import requests
from dateutil.parser import parse

PODMAN_URL = "http://localhost:8080"


def _url(path):
    return PODMAN_URL + "/v1.0.0/libpod" + path


def podman():
    binary = os.getenv("PODMAN_BINARY")
    if binary is None:
        binary = "bin/podman"
    return binary


def ctnr(path):
    r = requests.get(_url("/containers/json?all=true"))
    try:
        ctnrs = json.loads(r.text)
    except Exception as e:
        sys.stderr.write("Bad container response: {}/{}".format(r.text, e))
        raise e
    return path.format(ctnrs[0]["Id"])


class TestApi(unittest.TestCase):
    podman = None

    def setUp(self):
        super().setUp()
        if TestApi.podman.poll() is not None:
            sys.stderr.write("podman service returned {}", TestApi.podman.returncode)
            sys.exit(2)
        requests.get(_url("/images/create?fromSrc=quay.io%2Flibpod%2Falpine%3Alatest"))
        # calling out to podman is easier than the API for running a container
        subprocess.run(
            [podman(), "run", "alpine", "/bin/ls"],
            check=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        )

    @classmethod
    def setUpClass(cls):
        super().setUpClass()

        TestApi.podman = subprocess.Popen(
            [
                podman(),
                "system",
                "service",
                "tcp:localhost:8080",
                "--log-level=debug",
                "--time=0",
            ],
            shell=False,
            stdin=subprocess.DEVNULL,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        )
        time.sleep(2)

    @classmethod
    def tearDownClass(cls):
        TestApi.podman.terminate()
        stdout, stderr = TestApi.podman.communicate(timeout=0.5)
        if stdout:
            print("\nService Stdout:\n" + stdout.decode("utf-8"))
        if stderr:
            print("\nService Stderr:\n" + stderr.decode("utf-8"))

        if TestApi.podman.returncode > 0:
            sys.stderr.write("podman exited with error code {}\n".format(TestApi.podman.returncode))
            sys.exit(2)

        return super().tearDownClass()

    def test_info(self):
        r = requests.get(_url("/info"))
        self.assertEqual(r.status_code, 200)
        self.assertIsNotNone(r.content)
        _ = json.loads(r.text)

    def test_events(self):
        r = requests.get(_url("/events?stream=false"))
        self.assertEqual(r.status_code, 200, r.text)
        self.assertIsNotNone(r.content)
        for line in r.text.splitlines():
            obj = json.loads(line)
            # Actor.ID is uppercase for compatibility
            _ = obj["Actor"]["ID"]

    def test_containers(self):
        r = requests.get(_url("/containers/json"), timeout=5)
        self.assertEqual(r.status_code, 200, r.text)
        obj = json.loads(r.text)
        self.assertEqual(len(obj), 0)

    def test_containers_all(self):
        r = requests.get(_url("/containers/json?all=true"))
        self.assertEqual(r.status_code, 200, r.text)
        self.validateObjectFields(r.text)

    def test_inspect_container(self):
        r = requests.get(_url(ctnr("/containers/{}/json")))
        self.assertEqual(r.status_code, 200, r.text)
        obj = self.validateObjectFields(r.content)
        _ = parse(obj["Created"])

    def test_stats(self):
        r = requests.get(_url(ctnr("/containers/{}/stats?stream=false")))
        self.assertIn(r.status_code, (200, 409), r.text)
        if r.status_code == 200:
            self.validateObjectFields(r.text)

    def test_delete_containers(self):
        r = requests.delete(_url(ctnr("/containers/{}")))
        self.assertEqual(r.status_code, 204, r.text)

    def test_stop_containers(self):
        r = requests.post(_url(ctnr("/containers/{}/start")))
        self.assertIn(r.status_code, (204, 304), r.text)

        r = requests.post(_url(ctnr("/containers/{}/stop")))
        self.assertIn(r.status_code, (204, 304), r.text)

    def test_start_containers(self):
        r = requests.post(_url(ctnr("/containers/{}/stop")))
        self.assertIn(r.status_code, (204, 304), r.text)

        r = requests.post(_url(ctnr("/containers/{}/start")))
        self.assertIn(r.status_code, (204, 304), r.text)

    def test_restart_containers(self):
        r = requests.post(_url(ctnr("/containers/{}/start")))
        self.assertIn(r.status_code, (204, 304), r.text)

        r = requests.post(_url(ctnr("/containers/{}/restart")), timeout=5)
        self.assertEqual(r.status_code, 204, r.text)

    def test_resize(self):
        r = requests.post(_url(ctnr("/containers/{}/resize?h=43&w=80")))
        self.assertIn(r.status_code, (200, 409), r.text)
        if r.status_code == 200:
            self.assertIsNone(r.text)

    def test_attach_containers(self):
        r = requests.post(_url(ctnr("/containers/{}/attach")))
        self.assertIn(r.status_code, (101, 409), r.text)

    def test_logs_containers(self):
        r = requests.get(_url(ctnr("/containers/{}/logs?stdout=true")))
        self.assertEqual(r.status_code, 200, r.text)

    def test_post_create(self):
        self.skipTest("TODO: create request body")
        r = requests.post(_url("/containers/create?args=True"))
        self.assertEqual(r.status_code, 200, r.text)
        json.loads(r.text)

    def test_commit(self):
        r = requests.post(_url(ctnr("/commit?container={}")))
        self.assertEqual(r.status_code, 200, r.text)
        self.validateObjectFields(r.text)

    def test_images(self):
        r = requests.get(_url("/images/json"))
        self.assertEqual(r.status_code, 200, r.text)
        self.validateObjectFields(r.content)

    def test_inspect_image(self):
        r = requests.get(_url("/images/alpine/json"))
        self.assertEqual(r.status_code, 200, r.text)
        obj = self.validateObjectFields(r.content)
        _ = parse(obj["Created"])

    def test_delete_image(self):
        r = requests.delete(_url("/images/alpine?force=true"))
        self.assertEqual(r.status_code, 200, r.text)
        json.loads(r.text)

    def test_pull(self):
        r = requests.post(_url("/images/pull?reference=alpine"), timeout=5)
        self.assertEqual(r.status_code, 200, r.text)
        json.loads(r.text)

    def test_search(self):
        # Had issues with this test hanging when repositories not happy
        def do_search():
            r = requests.get(_url("/images/search?term=alpine"), timeout=5)
            self.assertEqual(r.status_code, 200, r.text)
            json.loads(r.text)

        search = Process(target=do_search)
        search.start()
        search.join(timeout=10)
        self.assertFalse(search.is_alive(), "/images/search took too long")

    def test_ping(self):
        r = requests.get(PODMAN_URL + "/_ping")
        self.assertEqual(r.status_code, 200, r.text)

        r = requests.head(PODMAN_URL + "/_ping")
        self.assertEqual(r.status_code, 200, r.text)

        r = requests.get(_url("/_ping"))
        self.assertEqual(r.status_code, 200, r.text)

        r = requests.get(_url("/_ping"))
        self.assertEqual(r.status_code, 200, r.text)


def validateObjectFields(self, buffer):
    objs = json.loads(buffer)
    if not isinstance(objs, dict):
        for o in objs:
            _ = o["Id"]
    else:
        _ = objs["Id"]
    return objs


if __name__ == "__main__":
    unittest.main()



import os
import random
import unittest

import requests
from .fixtures import APITestCase


class VolumeTestCase(APITestCase):
    def test_volume_crud(self):
        name = f"Volume_{random.getrandbits(160):x}"

        ls = requests.get(self.podman_url + "/v1.40/volumes")
        self.assertEqual(ls.status_code, 200, ls.text)

        # See https://docs.docker.com/engine/api/v1.40/#operation/VolumeList
        required_keys = (
            "Volumes",
            "Warnings",
        )

        volumes = ls.json()
        self.assertIsInstance(volumes, dict)
        for key in required_keys:
            self.assertIn(key, volumes)

        create = requests.post(self.podman_url + "/v1.40/volumes/create", json={"Name": name})
        self.assertEqual(create.status_code, 201, create.text)

        # See https://docs.docker.com/engine/api/v1.40/#operation/VolumeCreate
        # and https://docs.docker.com/engine/api/v1.40/#operation/VolumeInspect
        required_keys = (
            "Name",
            "Driver",
            "Mountpoint",
            "Labels",
            "Scope",
            "Options",
        )

        volume = create.json()
        self.assertIsInstance(volume, dict)
        for k in required_keys:
            self.assertIn(k, volume)
        self.assertEqual(volume["Name"], name)

        inspect = requests.get(self.podman_url + f"/v1.40/volumes/{name}")
        self.assertEqual(inspect.status_code, 200, inspect.text)

        volume = inspect.json()
        self.assertIsInstance(volume, dict)
        for k in required_keys:
            self.assertIn(k, volume)

        rm = requests.delete(self.podman_url + f"/v1.40/volumes/{name}")
        self.assertEqual(rm.status_code, 204, rm.text)

        # recreate volume with data and then prune it
        r = requests.post(self.podman_url + "/v1.40/volumes/create", json={"Name": name})
        self.assertEqual(create.status_code, 201, create.text)

        create = r.json()
        with open(os.path.join(create["Mountpoint"], "test_prune"), "w") as file:
            file.writelines(["This is a test\n", "This is a good test\n"])

        prune = requests.post(self.podman_url + "/v1.40/volumes/prune")
        self.assertEqual(prune.status_code, 200, prune.text)

        payload = prune.json()
        self.assertIn(name, payload["VolumesDeleted"])
        self.assertGreater(payload["SpaceReclaimed"], 0)

    def test_volume_label(self):
        name = f"Volume_{random.getrandbits(160):x}"
        expected = {
            "Production": "False",
            "Database": "Foxbase",
        }

        create = requests.post(
            self.podman_url + "/v4.0.0/libpod/volumes/create",
            json={"name": name, "label": expected},
        )
        self.assertEqual(create.status_code, 201, create.text)

        inspect = requests.get(self.podman_url + f"/v4.0.0/libpod/volumes/{name}/json")
        self.assertEqual(inspect.status_code, 200, inspect.text)

        volume = inspect.json()
        self.assertIn("Labels", volume)
        self.assertNotIn("Label", volume)
        self.assertDictEqual(expected, volume["Labels"])

    def test_volume_labels(self):
        name = f"Volume_{random.getrandbits(160):x}"
        expected = {
            "Production": "False",
            "Database": "Foxbase",
        }

        create = requests.post(
            self.podman_url + "/v4.0.0/libpod/volumes/create",
            json={"name": name, "labels": expected},
        )
        self.assertEqual(create.status_code, 201, create.text)

        inspect = requests.get(self.podman_url + f"/v4.0.0/libpod/volumes/{name}/json")
        self.assertEqual(inspect.status_code, 200, inspect.text)

        volume = inspect.json()
        self.assertIn("Labels", volume)
        self.assertDictEqual(expected, volume["Labels"])

    def test_volume_label_override(self):
        name = f"Volume_{random.getrandbits(160):x}"
        create = requests.post(
            self.podman_url + "/v4.0.0/libpod/volumes/create",
            json={
                "Name": name,
                "Label": {
                    "Database": "dbase",
                },
                "Labels": {
                    "Database": "sqlserver",
                },
            },
        )
        self.assertEqual(create.status_code, 201, create.text)

        inspect = requests.get(self.podman_url + f"/v4.0.0/libpod/volumes/{name}/json")
        self.assertEqual(inspect.status_code, 200, inspect.text)

        volume = inspect.json()
        self.assertIn("Labels", volume)
        self.assertNotIn("Label", volume)
        self.assertDictEqual({"Database": "sqlserver"}, volume["Labels"])


if __name__ == "__main__":
    unittest.main()
